'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.swaggerToReactSchema = swaggerToReactSchema;
exports.createReactSchemaSkeleton = createReactSchemaSkeleton;
var DATE_PATTERN = "^[0-9]{4}-(0[1-9]|(1[0|1|2]))-((0[1-9])|((1|2)[0-9])|(30|31))$";

function swaggerToReactSchema(swagger) {
  var schema = swagger;
  try {
    var result = createReactSchemaSkeleton(schema.title);
    var properties = schema.type === 'array' ? schema.items.properties : schema.properties;
    var keys = Object.keys(properties);
    var count = 2;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = keys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var key = _step.value;

        var item = properties[key];
        if (item.type === 'object' || item.type === 'array' && item.items.type === 'object') {
          var questionSet = createQuestionSet('question-set-' + count, properties[key], key);
          result.questionSets.push(questionSet);

          //question panel
          var qrQuestionSet = {
            "index": count,
            "questionSetId": 'question-set-' + count
          };

          result.questionPanels[0].questionSets.push(qrQuestionSet);
          count++;
        } else {
          var requiredItems = schema.required;
          var isRequired = requiredItems && requiredItems.indexOf(key) >= 0;
          var question = createQuestion(key, properties[key], isRequired);
          result.questionSets[0].questions.push(question);
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return result;
  } catch (ex) {
    console.error("error while parssing", ex);
    return null;
  }
}

function createReactSchemaSkeleton(title, displayTitleField) {
  displayTitleField = displayTitleField || "ProductName";
  return {
    "showErrorSummary": true,
    "name": title,
    "displayTitleField": displayTitleField,
    "summaryErrorMessage": "There are errors in your input. Please correct them and submit again.",
    "classes": {
      "questionPanelHeaderContainer": "question-panel-header",
      "questionSetHeaderContainer": "question-set-header",
      "errorContainer": "error-container",
      "input": "form-control flat",
      "question": "form-group",
      "backButton": "pull-left",
      "errorMessage": "error-input",
      "buttonBar": "button-bar",
      "radioListItem": "radio"
    },
    "formPanels": [{
      "index": 1,
      "panelId": "my-panel"
    }],
    "questionPanels": [{
      "panelId": "my-panel",
      "disableDefaultButton": true,
      "panelHeader": title + ' Definition',
      "action": {
        "default": {
          "action": "SUBMIT"
        }
      },
      "button": {
        "text": "Send"
      },
      "questionSets": [{
        "index": 1,
        "questionSetId": "question-set-1"
      }]
    }],
    "questionSets": [{
      "questionSetId": "question-set-1",
      "questionSetHeader": "Product Summary",
      "collapsible": true,
      "questions": []
    }]
  };
}

function createQuestionSet(questionSetId, schema, key) {
  try {
    var questions = [];

    var properties = schema.type === 'array' ? schema.items.properties : schema.properties;

    questions.push(createQuestion(key, schema, schema.required));

    var result = {
      "questionSetId": questionSetId,
      "collapsible": true,
      "collapse": true,
      "questions": questions,
      "questionSetHeader": key.replace(/([A-Z])/g, ' $1').trim()
    };

    return result;
  } catch (ex) {
    console.log("coudn't create question set", questionSetId, schema, key, ex);
  }
}

function createQuestions(properties, requiredItems) {
  var result = [];
  var requiredQuestions = requiredItems || [];
  var keys = Object.keys(properties);
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = keys[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var key = _step2.value;

      var isRequired = requiredItems && requiredItems.indexOf(key) >= 0;
      var question = createQuestion(key, properties[key], isRequired);
      result.push(question);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return result;
}

function createQuestion(key, item, isRequired) {
  var description = item.description || key;

  var result = {
    "questionId": key,
    "input": {
      "type": "materialTextField",
      "required": Boolean(isRequired),
      "props": {}
    }
  };

  if (item.type === 'string' || item.type === 'integer' || item.type === 'number' || item.type === 'boolean') {
    result.input.props = {
      "fullWidth": true,
      "hintText": description,
      "floatingLabelText": key
    };
  }

  if (item.type === 'string') {
    if (item.enum) {
      //dropdown
      result.input.options = item.enum.map(function (enumItem) {
        return {
          "text": enumItem,
          "value": enumItem
        };
      });
      result.input.type = 'MaterialSelectField';
    } else {
      populateInputType(result.input, item);
    }
  } else if (item.type === 'integer' || item.type === 'number') {
    result.input.type = 'materialTextField';
    result.input.props.type = 'number';
  } else if (item.type === 'boolean') {
    result.question = key + '?';
    result.input.type = 'radioOptionsInput';
    result.input.options = [{
      "text": "Yes",
      "value": true,
      "validations": [{
        "type": "isLength",
        "params": [1]
      }]
    }, {
      "text": "No",
      "value": false
    }];
  } else if (item.type === 'array') {
    if (item.items.type === 'object') {
      result.input.props.description = 'A collection of ' + key.replace(/([A-Z])/g, ' $1').trim();
      result.input.type = 'ArrayInput';
      var elements = { questions: [] };
      elements.questions = createQuestions(item.items.properties, item.items.required);
      result.input.props.elements = elements;
    } else {
      if (item.items.enum) {
        populateMultiSelect(result, item);
      } else {
        populateTextFieldWithArray(result, item, key);
      }
    }
  } else if (item.type === 'object') {
    //snake case to spaces in title
    result.question = key.replace(/([A-Z])/g, ' $1').trim();
    result.input.type = 'ObjectInput';
    var _elements = { questions: [] };
    _elements.questions = createQuestions(item.properties, item.required);
    result.input.props.elements = _elements;
  } else {
    console.error("unknown: ", item);
    result.input.type = null;
  }
  return result;
}

function populateInputType(input, swaggerItem) {
  var pattern = swaggerItem.pattern;
  if (pattern && pattern === DATE_PATTERN) {
    input.type = 'MaterialDatePicker';
  } else {
    input.type = 'materialTextField';
  }
}

function populateTextFieldWithArray(item, schema, key) {
  item.input.type = 'materialTextField';
  item.input.props = {
    "valueType": "array",
    "fullWidth": true,
    "hintText": schema.items.description,
    "floatingLabelText": key
  };
}

function populateMultiSelect(item, schema) {
  item.input.type = 'MultiSelect';
  item.input.placeholder = schema.items.description;
  if (schema.items.enum) {
    item.input.options = schema.items.enum.map(function (option) {
      return {
        "text": option,
        "value": option
      };
    });
  }

  item.input.props = {
    "multi": true,
    "simpleValue": true
  };

  if (schema.minItems) {
    item.validations = [{
      "type": "isLength",
      "params": [schema.minItems]
    }];
  }
}